---------------------------------------------------------------------------------------------
-- Title: w33zl's Merchant															 -
-- Author: @w33zl																 -
-- Description: 															 		 -
---------------------------------------------------------------------------------------------
-- This Add-on is not created by, affiliated with or sponsored by ZeniMax 	 -
-- Media Inc. or its affiliates. The Elder Scrolls® and related logos are 	 	 -
-- registered trademarks or trademarks of ZeniMax Media Inc. in the United	 - 
-- States and/or other countries. All rights reserved.						 	 -
---------------------------------------------------------------------------------------------

wMerchant = WZ_Chain(ADDONS:Create("wMerchant", "0.9.6.3", "w33zl", "Displays guild sales statistics"))
	-- :EnableSettings(true) -- Enable saved variables automatically, include global settings
.__END

-- Prepare global item database
WMDB = {}

-- Prepare submodules
wMerchant.PlayerSales = { Parent = wMerchant }
wMerchant.UI = { Parent = wMerchant }
--wMerchantUI = {}

-- Create local shortcuts
local WM = wMerchant
local PS = wMerchant.PlayerSales
local UI = wMerchant.UI


function WM:OnLoad()
	
	self:InitSettings()

	if self:InitDatabase() then
		UI:Init()
		self:InitPlayerSales()
	end--if
	

	ZO_CreateStringId("SI_BINDING_NAME_SHOW_WM_WINDOW","Toggle w33zl's Merchant window")
	ZO_CreateStringId("SHOW_WM_WINDOW","Toggle w33zl's Merchant window")

	if self.InitDataAggregator then self:InitDataAggregator() end
end

function wMerchant:OnReady()
	if self.IsSampleDB then
		warn(wMerchant.Addon.Name .. ": Sample database loaded, please install ESO Safe to get up-to-date data")
	elseif self.IsOldDB then
		warn(wMerchant.Addon.Name .. ": Database loaded from old location, consider moving it to prevent provblems (see addon-page on ESOUI for more details)")
	end
end

function wMerchant:InitSettings()
	local defaultSettings = 
	{
		WindowLeft = 170,
		WindowTop  = 95,
		IsFirstRun = true,
		AutoOpenOnLogin = false,
		EnabledDataAggregator = true,
		KeepFocusAfterSearch = false
	}

	wMerchant.Settings = ZO_SavedVars:NewAccountWide("Settings", 2, nil, defaultSettings)
end--function


--- Initialize database
function wMerchant:InitDatabase()

	-- d("wMerchant_Data: " .. tostring(wMerchant_Data))
	-- d("wMerchantDB: " .. tostring(wMerchantDB))
	-- d("wMerchant_Sample: " .. tostring(wMerchant_Sample))

	if wMerchant_Data then
		--d("New db was found")
		WMDB = wMerchant_Data -- load new path database

		-- if wMerchantDB then
		-- 	d("Old db was also found")
		-- else
		-- 	d("No old db was found")
		-- end

		wMerchantDB = nil -- make sure we are not holding two large tables in memory, remove old database (if any)
	elseif wMerchantDB then
		-- d("Only old db was found")
		
		WMDB = wMerchantDB -- load "old" path database
		self.IsOldDB = true
	elseif wMerchant_Sample then
		--d("Sample database loaded, please install ESO Safe to get up-to-date data")
		
		WMDB = wMerchant_Sample -- load sample database
		self.IsSampleDB = true
	end

	if WMDB == nil or 
		WMDB["SCHEMA"] == nil or 
		WMDB["ITEM_DATA"] == nil then
		err(wMerchant.Addon.Name .. ": Failed to load database")
		return
	end

	-- Cleanup old references (prevent holding ref to unused resources)
	wMerchant_Data = nil
	wMerchantDB = nil
	wMerchant_Sample = nil
	
	-- Get shortcuts for items, prices etc
	WMDB.Items = WMDB.ITEM_DATA
	WMDB.Prices = WMDB.PRICE_DATA
	WMDB.PlayerSales = WMDB.PLAYER_SALES

	return true
end--function


function wMerchant.MoveStop()
	wMerchant.Settings.WindowLeft = wMerchantDisplay:GetLeft()
	wMerchant.Settings.WindowTop = wMerchantDisplay:GetTop()
end


function wMerchant.Update()
	

end




-- Next we create a function that will initialize our addon
function wMerchant:Initialize()


  	


--d(_G["LUMINARY"])
--d(LUMINARY)


	-- delayed load
	--zo_callLater(function() wMerchant.Refresh() end, 1500)


	
	--wMerchant.InitUI()
	--UI:InitPlayerSales()
	
end


function table.length(T)
	return tablelength(T)
end

function tablelength(T)
  local count = 0
  for _ in pairs(T) do count = count + 1 end
  return count
end

function format_thousand(v)
    --local s = string.format("%d", v)
    local s = string.format("%d", math.floor(v))
    local pos = string.len(s) % 3
    if pos == 0 then pos = 3 end
    return string.sub(s, 1, pos)
    .. string.gsub(string.sub(s, pos+1), "(...)", ",%1")
end

-- from sam_lie
-- Compatible with Lua 5.0 and 5.1.
-- Disclaimer : use at own risk especially for hedge fund reports :-)

---============================================================
-- add comma to separate thousands
-- 
function comma_value(amount)
  local formatted = amount
  while true do  
    formatted, k = string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1,%2')
    if (k==0) then
      break
    end
  end
  return formatted
end

---============================================================
-- rounds a number to the nearest decimal places
--
function round(val, decimal)
  if (decimal) then
    return math.floor( (val * 10^decimal) + 0.5) / (10^decimal)
  else
    return math.floor(val+0.5)
  end
end

--===================================================================
-- given a numeric value formats output with comma to separate thousands
-- and rounded to given decimal places
--
--
function format_num(amount, decimal, prefix, neg_prefix)
  local str_amount,  formatted, famount, remain

  decimal = decimal or 2  -- default 2 decimal places
  neg_prefix = neg_prefix or "-" -- default negative sign

  famount = math.abs(round(amount,decimal))
  famount = math.floor(famount)

  remain = round(math.abs(amount) - famount, decimal)

        -- comma to separate the thousands
  formatted = comma_value(famount)

        -- attach the decimal portion
  if (decimal > 0) then
    remain = string.sub(tostring(remain),3)
    formatted = formatted .. "." .. remain ..
                string.rep("0", decimal - string.len(remain))
  end

        -- attach prefix string e.g '$' 
  formatted = (prefix or "") .. formatted 

        -- if value is negative then format accordingly
  if (amount<0) then
    if (neg_prefix=="()") then
      formatted = "("..formatted ..")"
    else
      formatted = neg_prefix .. formatted 
    end
  end

  return formatted
end

function comma_value2(n) -- credit http://richard.warburton.it
	local left,num,right = string.match(n,'^([^%d]*%d)(%d*)(.-)$')
	return left..(num:reverse():gsub('(%d%d%d)','%1,'):reverse())..right
end


function wMerchant.Refresh()
	local text = ""

	--d("Begin refresh")


	-- if WMDB then
	-- 	text = text .. "\nw33zl's Merchant database is ok!\n"

		-- if WMDB["ITEM_DATA"] then

	--d("|cFFFFFFw33zl's Merchant database is ok!\n\nHej!|r")




--		local itemData = WMDB["ITEM_DATA"]




		-- text = text .. "Item: " .. "44182_50_57_0" .. "\n"
		-- d(WMDB["ITEM_DATA"]["46014_0"])
		-- d(ZO_LinkHandler_CreateLink("Test",nil, ITEM_LINK_TYPE, "44182:57:50:0:1:2:3:4:5:6:7:8:9:10:11:5:0:0:1000:10000"))

		-- local link = "|H0:item:44182:25:50:0:0:0:0:0:0:0:0:0:0:0:0:5:0:0:10000:0|h|h"

		-- d(link)

-- 	end
-- end


-- if false then

-- 	local icon, sellPrice, meetsUsageRequirement, equipType, itemStyle = GetItemLinkInfo("|H0:item:5365:118:50:0:0:0:0:0:0:0:0:0:0:0:0:17:0:0:0:0|h|h")
-- 	local itemLink = "|H0:item:5365:118:50:0:0:0:0:0:0:0:0:0:0:0:0:17:0:0:0:0|h|h"
-- 	local itemName = zo_strformat(SI_TOOLTIP_ITEM_NAME, itemLink)

-- 	local itemName2 = string.gsub(itemName, "of", "hej")

-- 	text = text .. "Item: " .. sellPrice .. "\n"
-- 	text = text .. "Item: " .. "|H0:item:5365:118:50:0:0:0:0:0:0:0:0:0:0:0:0:17:0:0:0:0|h|h" .. "\n"
-- 	text = text .. "Item: " .. itemName .. "\n"
-- 	text = text .. "Item: " .. "|t16:16:EsoUI/Art/currency/currency_gold.dds|t"  .. "\n"

-- end



	--wMerchantDisplaySearchBox:SetText("lorem")

if false then	
	wMerchantDisplayTB:SetLinkEnabled(true)
	wMerchantDisplayTB:SetMouseEnabled(true)

		

		wMerchantDisplayTB:AddMessage("text", 255, 255, 255)
		wMerchantDisplayTB:AddMessage("text2\n", 255, 255, 255)

		wMerchantDisplayTB:AddMessage("text3\n", 255, 255, 255, 100)

		wMerchantDisplayTB:AddMessage(itemName3 .. "\n", 255, 255, 255)

	-- wMerchantDisplayTB:ShowFadedLines()
end



	
	-- --local text = ""
	-- for level=20,20,10 do
	--     for quality=188,265 do

	--     	local link = ZO_LinkHandler_CreateLink("Test " .. quality .. ":" .. level, nil, ITEM_LINK_TYPE, "45336:" .. quality .. ":" .. level .. ":0:0:0:0:0:0:0:0:0:0:0:0:5:0:0:0:0 ") -- Trash 100% more
	--     	local x1, x2 = ZOX_ItemHandler_GetLevelAndColor(quality, level)
	--     	d(link .. " > " .. (x1 or "n/a") .. ", " .. (x2 or "n/a"))

	--     end
	-- end

	

-- wMerchantDisplayTest:SetText(text)


	-- Update chat
	-- ZO_ChatWindowTextEntryEditBox:SetText(ZO_ChatWindowTextEntryEditBox:GetText() .. string.gsub(itemLink, "|H0", "|H1"))
	-- ZO_ChatWindowTextEntryEditBox:SetText("")


	-- local link = ZO_LinkHandler_CreateLink("My Item " .. 48 .. ":" .. 1, nil, ITEM_LINK_TYPE, "45336:" .. 48 .. ":" .. 1 .. ":0:0:0:0:0:0:0:0:0:0:0:0:5:0:0:0:0 ") -- Trash 100% more
	-- wMerchant:ShowToolTip(link, wMerchantDisplayPlayerSales)


end 

function wMerchant:ShowToolTip(itemName, itemButton)
  self.itemToolTip = PopupTooltip
  self.itemToolTip:ClearAnchors()
  self.itemToolTip:ClearLines()
  self.itemToolTip:SetLink(itemName)
  self.itemToolTip:SetHidden(false)
  self.itemToolTip:SetAnchor(LEFT, itemButton, RIGHT, -5, 0)
end

function wMerchant.LinkClicked(link, button, text, color, linkType)
	-- CHAT_SYSTEM:AddMessage("Link Clicked")
	--d("|H" .. button .. "|h" .. text .. "|h")
	-- d(text)
	wMerchant:ShowToolTip("|H" .. button .. "|h" .. text .. "|h", wMerchantDisplay)
    --ZO_ChatSystem_OnLinkClicked()
end
 
-- Then we create an event handler function which will be called when the "addon loaded" event
-- occurs. We'll use this to initialize our addon after all of its resources are fully loaded.
function wMerchant.OnAddOnLoaded(event, addonName)
  -- The event fires each time *any* addon loads - but we only care about when our own addon loads.
  if addonName == wMerchant.Addon.Name then
    wMerchant:Initialize()
  end
end


function wMerchant.Log(str)
	warn(str)
end

function wMerchant.Debug(str)																		-- It comes in handy for an addon to have its own debug function. Helps avoid global bleeds,
	if dd.Settings.Toggles.tDebug then																	-- but also allows customization and control.
		d(str)
	else
		return
	end
end

function wMerchant.Show()
	wMerchantDisplay:SetHidden(false)
	wMerchantDisplaySearchBox:TakeFocus()

end
function wMerchant.Toggle()
	if wMerchantDisplay:IsHidden() then
		wMerchantDisplay:SetHidden(false)
	else
		wMerchantDisplay:SetHidden(true)
	end
end


function wMerchant.LuaMemory()																			-- Neat little script/call from Garkin. Displays memory used by Lua
	d(math.ceil(collectgarbage("count")).." KB")												-- http://www.esoui.com/forums/showthread.php?t=2067&highlight=Lua+memory+limit
end

function wMerchant.SearchBox_TextChanged()
	--function(self) d(wMerchantDisplaySearchBox:GetText()) end
	local searchText = wMerchantDisplaySearchBox:GetText()

	--wMerchant.DoSearch(searchText)
end

-- function wMerchant.DoSearch(searchText)
-- 	d("Show")
-- 	wMerchantDisplaySearchBox:TakeFocus()
-- end


--- Parses a ItemUID string into actual vales (base itemID, level, quality and enchantmentID)
function WM.ParseItemUID(itemKey)
	--d("mtype: " .. type(itemKey))
	local itemKeyPattern = '(%d+):(%d+):(%d+):(%d+)'
	local itemID, level, quality, enchantmentID = string.match(itemKey, itemKeyPattern)
	return itemID, level, quality, enchantmentID
end

function WM.GetLinkIconTexture(link, size)
	local icon, _, _, _, _ = GetItemLinkInfo(link)

	size = size or 16

	return "|t" .. size .. ":" .. size .. ":" .. icon .. "|t "
end--function

function wMerchant:GetItemLevelText(quality, level)
	local displayLevel, _ = ZOX_ItemHandler_GetLevelAndColor(quality, level)


	local output = string.format("|c82807F[%s]|r  ", (displayLevel or "n/a"))

	if displayLevel then
		if string.len(displayLevel) == 1 then
			output = "     " .. output
		elseif string.len(displayLevel) == 2 then
			output = "   " .. output
		elseif string.len(displayLevel) == 3 then
			output = " " .. output
		end
	else
		output = " " .. output
	end

	return output
end

function wMerchant:GetItemLink(itemID, level, quality, enchantmentID)
	local itemLinkPattern = ":item:%i:%i:%i:%i:%i:%i:0:0:0:0:0:0:0:0:0:5:0:0:10000:0"
	local enchantment2 = 0
	local enchantment3 = 0

	if enchantmentID ~= 0 then
		enchantment2 = quality
		enchantment3 = level
	end



	return "|H0" .. string.format(itemLinkPattern, itemID, quality, level, enchantmentID, enchantment2, enchantment3) .. "|h|h"
end

function wMerchant.Load()
	d("det fungerar")
end

--wMerchant.TestIt()


--- EVENTS


-- Finally, we'll register our eventsent handler function to be called when the proper event occurs.
EVENT_MANAGER:RegisterForEvent(wMerchant.Addon.Name, EVENT_ADD_ON_LOADED, wMerchant.OnAddOnLoaded)

EVENT_MANAGER:RegisterForEvent(wMerchant.Addon.Name, EVENT_PLAYER_ACTIVATED, function() 
	--d("Player activated")
	--WM:Load()
	wMerchant:Refresh()
end )



--ZO_PreHookHandler(wMerchantDisplaySearchBox, "OnTextChanged", wMerchant.SearchBox_TextChanged)


SLASH_COMMANDS["/rl"] = function() ReloadUI("ingame") end
SLASH_COMMANDS["/mem"] = function() wMerchant.LuaMemory() end
--SLASH_COMMANDS["/foo update"] = function() wMerchant.Refresh() end
SLASH_COMMANDS["/merchant"] = function() wMerchant.Show() end
--SLASH_COMMANDS["/wm show"] = function() wMerchant.Show() end
SLASH_COMMANDS["/wm"] = function(text) 
	if not text or text == "" then
		wMerchant.Toggle() 
	elseif text == "player" then
		wMerchant.ShowPlayerSales() 
	elseif text == "prices" then
		wMerchant.ShowSales() 
	end
end
